##
## Author......: Luis Gustavo Nardin
## Last Change.: 09/25/2016
##
library(colorRamps)
library(data.table)
library(ggplot2)
library(grid)
library(gridExtra)


###############
## PATHS
###############
baseDir <- "."
disease1Data <- "../data/disease1"
disease2Data <- "../data/disease2"
outputDir <- "../figures"

###############
## FUNCTIONS
###############
source("calcExpectedTime.R")


###############
## CASES
###############

xmn <- -100
xmx <- -100
ymn <- 115
ymx <- 115

###############
## CASE 1 - DISEASE 1
###############
filename <- "disease1-1"
data <- data.table(read.table(paste0(disease1Data, "/", filename,".csv"),
                              sep=";", header=TRUE))

maxh <- 365
pData <- data[which((h <= maxh))]

ymin <- 1 - max(pData[which(n == 2 & h == 365 & pI >= 0.3)]$rho)
ymax <- 1 - min(pData[which(n == 2 & h == 365 & pI >= 0.9)]$rho)

pce1 <- ggplot(pData[which((n == 0) & (i < 1))],
               aes(x=h, y=(1 - rho) * 100, fill=(i * 100))) +
  ggtitle(expression(paste("u"[S]*" = u"[R]*" > u"[P]))) +
  xlab("") +
  ylab(expression(paste("% Protection"))) +
  #ylab("") +
  xlim(0, maxh + 30) +
  geom_raster(interpolate=TRUE, stat="identity") +
  geom_contour(data=pData[which(n == 2)],
               aes(x=h, y=(1 - rho) * 100, z=pI),
               breaks=c(0.3, 0.5, 0.7, 0.9),
               color="black",
               linetype="solid",
               size=0.5) +
  geom_line(data=pData[which(n == 2)],
            alpha=0.05,
            size=1) +
  scale_y_continuous(limits = c(0, 100),
                     breaks = c(0, 25, 50, 75, 100),
                     labels = c("0%", "25%", "50%", "75%", "100%")) +
  scale_fill_gradientn(name = expression(paste("% Infectious (i)")),
                       limits = c(0, 100),
                       values = c(0.0, 0.05, 0.1, 0.2, 0.3, 0.5, 0.7, 1.0),
                       colors = blue2green2red(50),
                       labels = c("0%", "25%", "50%", "75%", "100%")) +
  theme(axis.title.x = element_text(colour='black', size=34, face='bold',
                                    margin=margin(t=0.5, unit = "cm")),
        axis.title.y = element_text(colour='black', size=34, face='bold',
                                    margin=margin(r=0.2, unit = "cm")),
        axis.text.x = element_text(colour='black', size=16, face='bold'),
        axis.text.y = element_text(colour='black', size=16, face='bold'),
        axis.line.x = element_line(colour='black', size=1, linetype='solid'),
        axis.line.y = element_line(colour='black', size=1, linetype='solid'),
        panel.background = element_rect(fill="transparent", colour=NA),
        panel.grid.minor = element_blank(),
        panel.grid.major = element_blank(),
        legend.position = "none",
        legend.title = element_text(colour="black", size=32, face="bold"),
        legend.text = element_text(colour="black", size=28, face="bold"),
        legend.key = element_rect(fill = "white"),
        legend.key.height=unit(2, "line"),
        plot.margin = unit(c(1, 0, 0, 1), "cm"),
        plot.title = element_text(color="black", size=40, face="bold"))

pce1 <- pce1 + annotation_custom(
  grob = textGrob(label = "A", hjust = 0,
                  gp = gpar(cex=4, fontface="bold")),
  ymin = ymn,
  ymax = ymx,
  xmin = xmn,
  xmax = xmx)

gtce1 <- ggplot_gtable(ggplot_build(pce1))
gtce1$layout$clip[gtce1$layout$name == "panel"] <- "off"


###############
## CASE 2 - DISEASE 1
###############
filename <- "disease1-0.97"
data <- data.table(read.table(paste0(disease1Data, "/", filename,".csv"),
                              sep=";", header=TRUE))

maxh <- 365
pData <- data[which((h <= maxh))]

ymin <- 1 - max(pData[which(n == 2 & h == 365 & pI >= 0.3)]$rho)
ymax <- 1 - min(pData[which(n == 2 & h == 365 & pI >= 0.9)]$rho)

pce2 <- ggplot(pData[which((n == 0) & (i < 1))],
               aes(x=h, y=(1 - rho) * 100, fill=(i * 100))) +
  ggtitle(expression(paste("u"[S]*" > u"[R]*" > u"[P]))) +
  xlab("") +
  ylab("") +
  xlim(0, maxh + 30) +
  geom_raster(interpolate=TRUE, stat="identity") +
  geom_contour(data=pData[which(n == 2)],
               aes(x=h, y=(1 - rho) * 100, z=pI),
               breaks=c(0.3, 0.5, 0.7, 0.9),
               color="black",
               linetype="solid",
               size=0.5) +
  geom_line(data=pData[which(n == 2)],
            alpha=0.05,
            size=1) +
  scale_y_continuous(limits = c(0, 100),
                     breaks = c(0, 25, 50, 75, 100),
                     labels = c("0%", "25%", "50%", "75%", "100%")) +
  scale_fill_gradientn(name = expression(paste("% Infectious (i)")),
                       limits = c(0, 100),
                       values = c(0.0, 0.05, 0.1, 0.2, 0.3, 0.5, 0.7, 1.0),
                       colors = blue2green2red(50),
                       labels = c("0%", "25%", "50%", "75%", "100%")) +
  theme(axis.title.x = element_text(colour='black', size=34, face='bold',
                                    margin=margin(t=0.5, unit = "cm")),
        axis.title.y = element_text(colour='black', size=34, face='bold',
                                    margin=margin(r=0.2, unit = "cm")),
        axis.text.x = element_text(colour='black', size=16, face='bold'),
        axis.text.y = element_text(colour='black', size=16, face='bold'),
        axis.line.x = element_line(colour='black', size=1, linetype='solid'),
        axis.line.y = element_line(colour='black', size=1, linetype='solid'),
        panel.background = element_rect(fill="transparent", colour=NA),
        panel.grid.minor = element_blank(),
        panel.grid.major = element_blank(),
        legend.position = "none",
        legend.title = element_text(colour="black", size=32, face="bold"),
        legend.text = element_text(colour="black", size=28, face="bold"),
        legend.key = element_rect(fill = "white"),
        legend.key.height=unit(2, "line"),
        plot.margin = unit(c(1, 0, 0, 0), "cm"),
        plot.title = element_text(color="black", size=40, face="bold"))

pce2 <- pce2 + annotation_custom(
  grob = textGrob(label = "B", hjust = 0,
                  gp = gpar(cex=4, fontface="bold")),
  ymin = ymn,
  ymax = ymx,
  xmin = xmn,
  xmax = xmx)

gtce2 <- ggplot_gtable(ggplot_build(pce2))
gtce2$layout$clip[gtce2$layout$name == "panel"] <- "off"


###############
## CASE 3 - DISEASE 1
###############
filename <- "disease1-0.95"
data <- data.table(read.table(paste0(disease1Data, "/", filename,".csv"),
                              sep=";", header=TRUE))

maxh <- 365
pData <- data[which((h <= maxh))]

ymin <- 1 - max(pData[which(n == 2 & h == 365 & pI >= 0.3)]$rho)
ymax <- 1 - min(pData[which(n == 2 & h == 365 & pI >= 0.9)]$rho)

pce3 <- ggplot(pData[which((n == 0) & (i < 1))],
               aes(x=h, y=(1 - rho) * 100, fill=(i * 100))) +
  ggtitle(expression(paste("u"[S]*" > u"[R]*" = u"[P]))) +
  xlab("") +
  ylab("") +
  xlim(0, maxh + 30) +
  geom_raster(interpolate=TRUE, stat="identity") +
  geom_contour(data=pData[which(n == 2)],
               aes(x=h, y=(1 - rho) * 100, z=pI),
               breaks=c(0.3, 0.5, 0.7, 0.9),
               color="black",
               linetype="solid",
               size=0.5) +
  geom_line(data=pData[which(n == 2)],
            alpha=0.05,
            size=1) +
  scale_y_continuous(limits = c(0, 100),
                     breaks = c(0, 25, 50, 75, 100),
                     labels = c("0%", "25%", "50%", "75%", "100%")) +
  scale_fill_gradientn(name = expression(paste("% Infectious (i)")),
                       limits = c(0, 100),
                       values = c(0.0, 0.05, 0.1, 0.2, 0.3, 0.5, 0.7, 1.0),
                       colors = blue2green2red(50),
                       labels = c("0%", "25%", "50%", "75%", "100%")) +
  theme(axis.title.x = element_text(colour='black', size=34, face='bold',
                                    margin=margin(t=0.5, unit = "cm")),
        axis.title.y = element_text(colour='black', size=34, face='bold',
                                    margin=margin(r=0.2, unit = "cm")),
        axis.text.x = element_text(colour='black', size=16, face='bold'),
        axis.text.y = element_text(colour='black', size=16, face='bold'),
        axis.line.x = element_line(colour='black', size=1, linetype='solid'),
        axis.line.y = element_line(colour='black', size=1, linetype='solid'),
        panel.background = element_rect(fill="transparent", colour=NA),
        panel.grid.minor = element_blank(),
        panel.grid.major = element_blank(),
        legend.position = "none",
        legend.title = element_text(colour="black", size=32, face="bold"),
        legend.text = element_text(colour="black", size=28, face="bold"),
        legend.key = element_rect(fill = "white"),
        legend.key.height=unit(2, "line"),
        plot.margin = unit(c(1, 0, 0, 0), "cm"),
        plot.title = element_text(color="black", size=40, face="bold"))

pce3 <- pce3 + annotation_custom(
  grob = textGrob(label = "C", hjust = 0,
                  gp = gpar(cex=4, fontface="bold")),
  ymin = ymn,
  ymax = ymx,
  xmin = xmn,
  xmax = xmx)

gtce3 <- ggplot_gtable(ggplot_build(pce3))
gtce3$layout$clip[gtce3$layout$name == "panel"] <- "off"


###############
## CASE 4 - DISEASE 1
###############
filename <- "disease1-0.9"
data <- data.table(read.table(paste0(disease1Data, "/", filename,".csv"),
                              sep=";", header=TRUE))

maxh <- 365
pData <- data[which((h <= maxh))]

ymin <- 1 - max(pData[which(n == 2 & h == 365 & pI >= 0.3)]$rho)
ymax <- 1 - min(pData[which(n == 2 & h == 365 & pI >= 0.9)]$rho)

pce4 <- ggplot(pData[which((n == 0) & (i < 1))],
               aes(x=h, y=(1 - rho) * 100, fill=(i * 100))) +
  ggtitle(expression(paste("u"[S]*" > u"[P]*" > u"[R]))) +
  xlab("") +
  ylab("") +
  xlim(0, maxh + 30) +
  geom_raster(interpolate=TRUE, stat="identity") +
  geom_contour(data=pData[which(n == 2)],
               aes(x=h, y=(1 - rho) * 100, z=pI),
               breaks=c(0.3, 0.5, 0.7, 0.9),
               color="black",
               linetype="solid",
               size=0.5) +
  geom_line(data=pData[which(n == 2)],
            alpha=0.05,
            size=1) +
  scale_y_continuous(limits = c(0, 100),
                     breaks = c(0, 25, 50, 75, 100),
                     labels = c("0%", "25%", "50%", "75%", "100%")) +
  scale_fill_gradientn(name = expression(paste("% Infectious (i)")),
                       limits = c(0, 100),
                       values = c(0.0, 0.05, 0.1, 0.2, 0.3, 0.5, 0.7, 1.0),
                       colors = blue2green2red(50),
                       labels = c("0%", "25%", "50%", "75%", "100%")) +
  theme(axis.title.x = element_text(colour='black', size=34, face='bold',
                                    margin=margin(t=0.5, unit = "cm")),
        axis.title.y = element_text(colour='black', size=34, face='bold',
                                    margin=margin(r=0.2, unit = "cm")),
        axis.text.x = element_text(colour='black', size=16, face='bold'),
        axis.text.y = element_text(colour='black', size=16, face='bold'),
        axis.line.x = element_line(colour='black', size=1, linetype='solid'),
        axis.line.y = element_line(colour='black', size=1, linetype='solid'),
        panel.background = element_rect(fill="transparent", colour=NA),
        panel.grid.minor = element_blank(),
        panel.grid.major = element_blank(),
        #legend.position = "none",
        legend.title = element_text(colour="black", size=32, face="bold"),
        legend.text = element_text(colour="black", size=28, face="bold"),
        legend.key = element_rect(fill = "white"),
        legend.key.height=unit(2, "line"),
        plot.margin = unit(c(1, 0, 0, 0), "cm"),
        plot.title = element_text(color="black", size=40, face="bold"))

pce4 <- pce4 + annotation_custom(
  grob = textGrob(label = "D", hjust = 0,
                  gp = gpar(cex=4, fontface="bold")),
  ymin = ymn,
  ymax = ymx,
  xmin = xmn,
  xmax = xmx)

gtce4 <- ggplot_gtable(ggplot_build(pce4))
gtce4$layout$clip[gtce4$layout$name == "panel"] <- "off"


###############
## CASE 1 - DISEASE 2
###############
filename <- "disease2-1"
data <- data.table(read.table(paste0(disease2Data, "/", filename,".csv"),
                              sep=";", header=TRUE))

maxh <- 365
pData <- data[which((h <= maxh))]

pcf1 <- ggplot(pData[which((n == 0) & (i < 1))],
               aes(x=h, y=(1 - rho) * 100, fill=(i * 100))) +
  ggtitle(expression(paste("u"[S]*" = u"[R]*" > u"[P]))) +
  xlab("") +
  ylab(expression(paste("% Protection"))) +
  xlim(0, maxh + 30) +
  geom_raster(interpolate=TRUE, stat="identity") +
  geom_contour(data=pData[which(n == 2)],
               aes(x=h, y=(1 - rho) * 100, z=pI),
               breaks=c(0.3, 0.5, 0.7, 0.9),
               color="black",
               linetype="solid",
               size=0.5) +
  geom_line(data=pData[which(n == 2)],
            alpha=0.05,
            size=1) +
  scale_y_continuous(limits = c(0, 100),
                     breaks = c(0, 25, 50, 75, 100),
                     labels = c("0%", "25%", "50%", "75%", "100%")) +
  scale_fill_gradientn(name = expression(paste("% Infectious (i)")),
                       limits = c(0, 100),
                       values = c(0.0, 0.05, 0.1, 0.2, 0.3, 0.5, 0.7, 1.0),
                       colors = blue2green2red(50),
                       labels = c("0%", "25%", "50%", "75%", "100%")) +
  theme(axis.title.x = element_text(colour='black', size=34, face='bold',
                                    margin=margin(t=0.5, unit = "cm")),
        axis.title.y = element_text(colour='black', size=34, face='bold',
                                    margin=margin(r=0.2, unit = "cm")),
        axis.text.x = element_text(colour='black', size=16, face='bold'),
        axis.text.y = element_text(colour='black', size=16, face='bold'),
        axis.line.x = element_line(colour='black', size=1, linetype='solid'),
        axis.line.y = element_line(colour='black', size=1, linetype='solid'),
        panel.background = element_rect(fill="transparent", colour=NA),
        panel.grid.minor = element_blank(),
        panel.grid.major = element_blank(),
        legend.position = "none",
        legend.title = element_text(colour="black", size=32, face="bold"),
        legend.text = element_text(colour="black", size=28, face="bold"),
        legend.key = element_rect(fill = "white"),
        legend.key.height=unit(2, "line"),
        plot.margin = unit(c(1, 0, 0, 1), "cm"),
        plot.title = element_text(color="black", size=40, face="bold"))

pcf1 <- pcf1 + annotation_custom(
  grob = textGrob(label = "E", hjust = 0,
                  gp = gpar(cex=4, fontface="bold")),
  ymin = ymn,
  ymax = ymx,
  xmin = xmn,
  xmax = xmx)

gtcf1 <- ggplot_gtable(ggplot_build(pcf1))
gtcf1$layout$clip[gtcf1$layout$name == "panel"] <- "off"


###############
## CASE 2 - DISEASE 2
###############
filename <- "disease2-0.97"
data <- data.table(read.table(paste0(disease2Data, "/", filename,".csv"),
                              sep=";", header=TRUE))

maxh <- 365
pData <- data[which((h <= maxh))]

pcf2 <- ggplot(pData[which((n == 0) & (i < 1))],
               aes(x=h, y=(1 - rho) * 100, fill=(i * 100))) +
  ggtitle(expression(paste("u"[S]*" > u"[R]*" > u"[P]))) +
  xlab("") +
  ylab("") +
  xlim(0, maxh + 30) +
  geom_raster(interpolate=TRUE, stat="identity") +
  geom_contour(data=pData[which(n == 2)],
               aes(x=h, y=(1 - rho) * 100, z=pI),
               breaks=c(0.3, 0.5, 0.7, 0.9),
               color="black",
               linetype="solid",
               size=0.5) +
  geom_line(data=pData[which(n == 2)],
            alpha=0.05,
            size=1) +
  scale_y_continuous(limits = c(0, 100),
                     breaks = c(0, 25, 50, 75, 100),
                     labels = c("0%", "25%", "50%", "75%", "100%")) +
  scale_fill_gradientn(name = expression(paste("% Infectious (i)")),
                       limits = c(0, 100),
                       values = c(0.0, 0.05, 0.1, 0.2, 0.3, 0.5, 0.7, 1.0),
                       colors = blue2green2red(50),
                       labels = c("0%", "25%", "50%", "75%", "100%")) +
  theme(axis.title.x = element_text(colour='black', size=34, face='bold',
                                    margin=margin(t=0.5, unit = "cm")),
        axis.title.y = element_text(colour='black', size=34, face='bold',
                                    margin=margin(r=0.2, unit = "cm")),
        axis.text.x = element_text(colour='black', size=16, face='bold'),
        axis.text.y = element_text(colour='black', size=16, face='bold'),
        axis.line.x = element_line(colour='black', size=1, linetype='solid'),
        axis.line.y = element_line(colour='black', size=1, linetype='solid'),
        panel.background = element_rect(fill="transparent", colour=NA),
        panel.grid.minor = element_blank(),
        panel.grid.major = element_blank(),
        legend.position = "none",
        legend.title = element_text(colour="black", size=32, face="bold"),
        legend.text = element_text(colour="black", size=28, face="bold"),
        legend.key = element_rect(fill = "white"),
        legend.key.height=unit(2, "line"),
        plot.margin = unit(c(1, 0, 0, 0), "cm"),
        plot.title = element_text(color="black", size=40, face="bold"))

pcf2 <- pcf2 + annotation_custom(
  grob = textGrob(label = "F", hjust = 0,
                  gp = gpar(cex=4, fontface="bold")),
  ymin = ymn,
  ymax = ymx,
  xmin = xmn,
  xmax = xmx)

gtcf2 <- ggplot_gtable(ggplot_build(pcf2))
gtcf2$layout$clip[gtcf2$layout$name == "panel"] <- "off"


###############
## CASE 3 - DISEASE 2
###############
filename <- "disease2-0.95"
data <- data.table(read.table(paste0(disease2Data, "/", filename,".csv"),
                              sep=";", header=TRUE))

maxh <- 365
pData <- data[which((h <= maxh))]

pcf3 <- ggplot(pData[which((n == 0) & (i < 1))],
               aes(x=h, y=(1 - rho) * 100, fill=(i * 100))) +
  ggtitle(expression(paste("u"[S]*" > u"[R]*" = u"[P]))) +
  xlab("") +
  ylab("") +
  xlim(0, maxh + 30) +
  geom_raster(interpolate=TRUE, stat="identity") +
  geom_contour(data=pData[which(n == 2)],
               aes(x=h, y=(1 - rho) * 100, z=pI),
               breaks=c(0.3, 0.5, 0.7, 0.9),
               color="black",
               linetype="solid",
               size=0.5) +
  geom_line(data=pData[which(n == 2)],
            alpha=0.05,
            size=1) +
  scale_y_continuous(limits = c(0, 100),
                     breaks = c(0, 25, 50, 75, 100),
                     labels = c("0%", "25%", "50%", "75%", "100%")) +
  scale_fill_gradientn(name = expression(paste("% Infectious (i)")),
                       limits = c(0, 100),
                       values = c(0.0, 0.05, 0.1, 0.2, 0.3, 0.5, 0.7, 1.0),
                       colors = blue2green2red(50),
                       labels = c("0%", "25%", "50%", "75%", "100%")) +
  theme(axis.title.x = element_text(colour='black', size=34, face='bold',
                                    margin=margin(t=0.5, unit = "cm")),
        axis.title.y = element_text(colour='black', size=34, face='bold',
                                    margin=margin(r=0.2, unit = "cm")),
        axis.text.x = element_text(colour='black', size=16, face='bold'),
        axis.text.y = element_text(colour='black', size=16, face='bold'),
        axis.line.x = element_line(colour='black', size=1, linetype='solid'),
        axis.line.y = element_line(colour='black', size=1, linetype='solid'),
        panel.background = element_rect(fill="transparent", colour=NA),
        panel.grid.minor = element_blank(),
        panel.grid.major = element_blank(),
        legend.position = "none",
        legend.title = element_text(colour="black", size=32, face="bold"),
        legend.text = element_text(colour="black", size=28, face="bold"),
        legend.key = element_rect(fill = "white"),
        legend.key.height=unit(2, "line"),
        plot.margin = unit(c(1, 0, 0, 0), "cm"),
        plot.title = element_text(color="black", size=40, face="bold"))

pcf3 <- pcf3 + annotation_custom(
  grob = textGrob(label = "G", hjust = 0,
                  gp = gpar(cex=4, fontface="bold")),
  ymin = ymn,
  ymax = ymx,
  xmin = xmn,
  xmax = xmx)

gtcf3 <- ggplot_gtable(ggplot_build(pcf3))
gtcf3$layout$clip[gtcf3$layout$name == "panel"] <- "off"


###############
## CASE 4 - DISEASE 2
###############
filename <- "disease2-0.9"
data <- data.table(read.table(paste0(disease2Data, "/", filename,".csv"),
                              sep=";", header=TRUE))

maxh <- 365
pData <- data[which((h <= maxh))]

pcf4 <- ggplot(pData[which((n == 0) & (i < 1))],
               aes(x=h, y=(1 - rho) * 100, fill=(i * 100))) +
  ggtitle(expression(paste("u"[S]*" > u"[P]*" > u"[R]))) +
  xlab("") +
  ylab("") +
  xlim(0, maxh + 30) +
  geom_raster(interpolate=TRUE, stat="identity") +
  geom_contour(data=pData[which(n == 2)],
               aes(x=h, y=(1 - rho) * 100, z=pI),
               breaks=c(0.3, 0.5, 0.7, 0.9),
               color="black",
               linetype="solid",
               size=0.5) +
  geom_line(data=pData[which(n == 2)],
            alpha=0.05,
            size=1) +
  scale_y_continuous(limits = c(0, 100),
                     breaks = c(0, 25, 50, 75, 100),
                     labels = c("0%", "25%", "50%", "75%", "100%")) +
  scale_fill_gradientn(name = expression(paste("% Infectious (i)")),
                       limits = c(0, 100),
                       values = c(0.0, 0.05, 0.1, 0.2, 0.3, 0.5, 0.7, 1.0),
                       colors = blue2green2red(50),
                       labels = c("0%", "25%", "50%", "75%", "100%")) +
  theme(axis.title.x = element_text(colour='black', size=34, face='bold',
                                    margin=margin(t=0.5, unit = "cm")),
        axis.title.y = element_text(colour='black', size=34, face='bold',
                                    margin=margin(r=0.2, unit = "cm")),
        axis.text.x = element_text(colour='black', size=16, face='bold'),
        axis.text.y = element_text(colour='black', size=16, face='bold'),
        axis.line.x = element_line(colour='black', size=1, linetype='solid'),
        axis.line.y = element_line(colour='black', size=1, linetype='solid'),
        panel.background = element_rect(fill="transparent", colour=NA),
        panel.grid.minor = element_blank(),
        panel.grid.major = element_blank(),
        #legend.position = "none",
        legend.title = element_text(colour="black", size=32, face="bold"),
        legend.text = element_text(colour="black", size=28, face="bold"),
        legend.key = element_rect(fill = "white"),
        legend.key.height=unit(2, "line"),
        plot.margin = unit(c(1, 0, 0, 0), "cm"),
        plot.title = element_text(color="black", size=40, face="bold"))

pcf4 <- pcf4 + annotation_custom(
  grob = textGrob(label = "H", hjust = 0,
                  gp = gpar(cex=4, fontface="bold")),
  ymin = ymn,
  ymax = ymx,
  xmin = xmn,
  xmax = xmx)

gtcf4 <- ggplot_gtable(ggplot_build(pcf4))
gtcf4$layout$clip[gtcf4$layout$name == "panel"] <- "off"

plot <- grid.arrange(gtce1, gtce2, gtce3, gtce4, 
                     gtcf1, gtcf2, gtcf3, gtcf4,
                     ncol=4, nrow=2,
                     layout_matrix=rbind(c(1, 2, 3, 4), c(5, 6, 7, 8)),
                     heights=c(1, 1), widths=c(0.25, 0.25, 0.25, 0.3),
                     bottom=textGrob(expression(paste("Planning Horizon (H)")),
                                     gp=gpar(fontsize=38,
                                             fontface="bold")),
                     left=textGrob(expression(paste("Disease 2                     Disease 1")),
                                   rot=90,
                                   gp=gpar(fontfamily="sans",
                                           fontsize=38,
                                           fontface="bold")))

ggsave(paste0(outputDir,"/figure4.pdf"), plot=plot,
       width=80, height=30, units="cm")
